unit PhoneBook.Resources.Contacts;

interface

uses
  System.Generics.Collections,
  WiRL.Core.Attributes,
  WiRL.http.Accept.MediaType,
  PhoneBook.Entities.Contacts;

type

{ TContactResource }

  [Path('contacts')]
  TContactResource = class
  private
  public
    // GET /contacts
    [GET, Produces(TMediaType.APPLICATION_JSON)]
    function GetContacts: TObjectList<TContact>;
    // POST /contacts
    [POST, Consumes(TMediaType.APPLICATION_JSON), Produces(TMediaType.APPLICATION_JSON)]
    function PostContact([BodyParam] ANewContact: TNewContact): TContact;
    // DELETE /contacts/1
    [DELETE, Path('{id}')]
    function DeleteContact([PathParam('id')] AContactID: Integer): TContact;
  end;

implementation

uses
  WiRL.Core.Exceptions,
  WiRL.Core.MessageBody.Default,
  WiRL.Core.Registry,
  PhoneBook.Samples.Contacts;

{ TContactResource }

function TContactResource.DeleteContact(AContactID: Integer): TContact;
begin
  Result := nil;
  for var LContact in FContacts do
    if LContact.ID = AContactID then
      begin
        Result := LContact.Clone;
        FContacts.Remove(LContact);
        Break;
      end;
  if not Assigned(Result) then
    raise EWiRLNotFoundException.Create('Contact not found');
end;

function TContactResource.GetContacts: TObjectList<TContact>;
begin
  Result := TObjectList<TContact>.Create(False);
  Result.AddRange(FContacts);
end;

function TContactResource.PostContact(ANewContact: TNewContact): TContact;
begin
  Inc(FContactID);
  Result := TContact.Create;
  Result.ID := FContactID;
  Result.Name := ANewContact.Name;
  Result.Phone := ANewContact.Phone;
  FContacts.Add(Result.Clone);
end;

initialization
  TWiRLResourceRegistry.Instance.RegisterResource<TContactResource>();

end.
