unit Organizer.Controllers.Tasks;

interface

uses
  System.Generics.Collections,
  MVCFramework,
  MVCFramework.Commons,
  Organizer.Entities.Task;

type

{ TTaskController }

  [MVCPath('/api')]
  TTaskController = class (TMVCController)
  public
    [MVCPath('/tasks')]
    [MVCHTTPMethod([httpGET])]
    function GetTasks: IMVCResponse;

    [MVCPath('/tasks')]
    [MVCHTTPMethod([httpPOST])]
    function PostTask([MVCFromBody] ATask: TTask): IMVCResponse;

    [MVCPath('/tasks/($ID)')]
    [MVCHTTPMethod([httpDELETE])]
    function DeleteTask(const ID: TGUID): IMVCResponse;

    [MVCPath('/tasks/($ID)')]
    [MVCHTTPMethod([httpPUT])]
    function UpdateTask(const ID: TGUID;
      [MVCFromBody] ATask: TTask): IMVCResponse;
  end;

implementation

uses
  System.SysUtils,
  Organizer.Helpers.Tasks;

var
  SampleTasks: TObjectList<TTask>;

{ TTaskController }

function TTaskController.DeleteTask(const ID: TGUID): IMVCResponse;
begin
  var LIndex := TTaskHelper.IndexOfTask(SampleTasks, ID);
  if LIndex < 0 then
  begin
    Render(HTTP_STATUS.NotFound);
    Exit;
  end;
  SampleTasks.Delete(LIndex);
  Render204NoContent;
end;

function TTaskController.GetTasks: IMVCResponse;
begin
  Result := MVCResponseBuilder
    .StatusCode(HTTP_STATUS.Created)
    .Body(SampleTasks, False)
    .Build;
end;

function TTaskController.PostTask(ATask: TTask): IMVCResponse;
begin
  var LTask := TTask.Create(TGUID.NewGuid, ATask.Description);
  SampleTasks.Add(LTask);
  Result := MVCResponseBuilder
    .StatusCode(HTTP_STATUS.Created)
    .Body(LTask, False)
    .Build;
end;

function TTaskController.UpdateTask(const ID: TGUID;
  [MVCFromBody] ATask: TTask): IMVCResponse;
begin
  var LIndex := TTaskHelper.IndexOfTask(SampleTasks, ID);
  if LIndex < 0 then
  begin
    Render(HTTP_STATUS.NotFound);
    Exit;
  end;
  var LTask := SampleTasks[LIndex];
  with LTask do
  begin
    Description := ATask.Description;
    Done := ATask.Done;
  end;
  Result := MVCResponseBuilder
    .StatusCode(HTTP_STATUS.OK)
    .Body(LTask, False)
    .Build;
end;

initialization

  SampleTasks := TTaskHelper.GenerateSamples;

finalization

  if Assigned(SampleTasks) then
    FreeAndNil(SampleTasks);

end.
