unit Organizer.Modules.App;

interface

uses
  System.SysUtils,
  System.Classes,
  Web.HTTPApp,
  MVCFramework;

type

{ TAppModule }

  TAppModule = class(TWebModule)
  private
    FMVC: TMVCEngine;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  WebModuleClass: TComponentClass = TAppModule;

implementation

{$R *.dfm}

uses
  System.IOUtils,
  MVCFramework.Commons,
  MVCFramework.Middleware.StaticFiles,
  Organizer.Controllers.Tasks;

{ TAppModule }

constructor TAppModule.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMVC := TMVCEngine.Create(Self,
    procedure(Config: TMVCConfig)
    begin
      Config.dotEnv := dotEnv;
      // session timeout (0 means session cookie)
      Config[TMVCConfigKey.SessionTimeout] :=
        dotEnv.Env('dmvc.session_timeout', '0');
      // default content-type
      Config[TMVCConfigKey.DefaultContentType] :=
        dotEnv.Env('dmvc.default.content_type',
        TMVCConstants.DEFAULT_CONTENT_TYPE);
      // default content charset
      Config[TMVCConfigKey.DefaultContentCharset] :=
        dotEnv.Env('dmvc.default.content_charset',
        TMVCConstants.DEFAULT_CONTENT_CHARSET);
      // unhandled actions are permitted?
      Config[TMVCConfigKey.AllowUnhandledAction] :=
        dotEnv.Env('dmvc.allow_unhandled_actions', 'false');
      // enables or not system controllers loading (available only from localhost requests)
      Config[TMVCConfigKey.LoadSystemControllers] :=
        dotEnv.Env('dmvc.load_system_controllers', 'true');
      // default view file extension
      Config[TMVCConfigKey.DefaultViewFileExtension] :=
        dotEnv.Env('dmvc.default.view_file_extension', 'html');
      // view path
      Config[TMVCConfigKey.ViewPath] := dotEnv.Env('dmvc.view_path',
        'templates');
      // use cache for server side views (use "false" in debug and "true" in production for faster performances
      Config[TMVCConfigKey.ViewCache] := dotEnv.Env('dmvc.view_cache', 'false');
      // Max Record Count for automatic Entities CRUD
      Config[TMVCConfigKey.MaxEntitiesRecordCount] :=
        dotEnv.Env('dmvc.max_entities_record_count',
        IntToStr(TMVCConstants.MAX_RECORD_COUNT));
      // Enable Server Signature in response
      Config[TMVCConfigKey.ExposeServerSignature] :=
        dotEnv.Env('dmvc.expose_server_signature', 'false');
      // Enable X-Powered-By Header in response
      Config[TMVCConfigKey.ExposeXPoweredBy] :=
        dotEnv.Env('dmvc.expose_x_powered_by', 'true');
      // Max request size in bytes
      Config[TMVCConfigKey.MaxRequestSize] :=
        dotEnv.Env('dmvc.max_request_size',
        IntToStr(TMVCConstants.DEFAULT_MAX_REQUEST_SIZE));
    end);

  //FMVC.AddController(TSampleController);
  FMVC.AddController(TTaskController);

  FMVC.AddMiddleware(TMVCStaticFilesMiddleware.Create('/static',
    TPath.Combine(ExtractFilePath(GetModuleName(HInstance)), 'www')));
end;

destructor TAppModule.Destroy;
begin
  if Assigned(FMVC) then
    FreeAndNil(FMVC);
  inherited Destroy;
end;

end.
