const apiUrl = "http://localhost:8080/api";

document.addEventListener('alpine:init', () => {

  Alpine.store("app", {

    async init() {
      const response = await fetch(apiUrl + "/tasks");
      const items = await response.json();
      this.tasks.items = items.data;
    },

    search: {
      description: '',

      reset () {
        this.description = '';
      }      
    },

    tasks: {

      items: [],
      newDescription: '',

      addNewTask() {
        fetch(apiUrl + "/tasks", {
          method: "POST",
          body: JSON.stringify({ description: this.newDescription, done: false }),
          headers: {"Content-type": "application/json; charset=UTF-8"}
        })
        .then(response => response.json())
        .then(body => {
          this.items.push(body.data);
          this.newDescription = '';
        }); 
      },

      deleteTask(task) {
        const index = this.items.indexOf(task);
        if (index < 0)
          return;
        fetch(`${apiUrl}/tasks/${task.id}`, { method: "DELETE" })
        .then(response => {
          if (response.status == 204)
            this.items.splice(index, 1);
        }); 
      },

      getItems(search) {
        if (!this.items)
          return [];
        const queryDescription = search.description.toLowerCase();
        return this.items.filter(t => t.description.toLowerCase().includes(queryDescription));
      },

      toggleDone(task) {        
        fetch(`${apiUrl}/tasks/${task.id}`, {
          method: "PUT",
          body: JSON.stringify({ description: task.description, done: !task.done }),
          headers: {"Content-type": "application/json; charset=UTF-8"}
        })
        .then(response => response.json())
        .then(body => {
          task.done = body.data.done;
        }); 
      },

      get countDone() {
        if (!this.items)
          return 0;
        return this.items.filter(t => t.done).length;
      },

      get countTotal() {
        if (!this.items)
          return 0;
        return this.items.length;
      },

      get hasTasks() {
        return this.items && this.items.length > 0;
      }

    } 

  });

});      
